/** View class
 *
 * @class View
 * 
 *
 * @author <a href="mailto:info@ilevia.com">info@ilevia.com</a>
 *
 *
 * @required <a href="http://jquery.com/">jQuery</a>
 * @required <a href="http://jqueryui.com/">jQuery UI</a>
 *
 */
"use strict";


function drawEllipse(ctx, centerX, centerY, width, height,color) {

    ctx.beginPath();

    ctx.lineWidth = 1;
    ctx.strokeStyle = '#444444';

    ctx.moveTo(centerX, centerY - height/2); // A1

    ctx.bezierCurveTo(
        centerX + width/2, centerY - height/2, // C1
        centerX + width/2, centerY + height/2, // C2
        centerX, centerY + height/2); // A2

    ctx.bezierCurveTo(
        centerX - width/2, centerY + height/2, // C3
        centerX - width/2, centerY - height/2, // C4
        centerX, centerY - height/2); // A1

    ctx.fillStyle = color;
    ctx.fill();
    ctx.closePath();
}

var View = {
    //Ready state
    ready: false,

    //Users
    addUser: function(oNode){
        var oUser = oNode.getData();
        //var $menu = $("[eve-manager-menu='users'] .users-list");
        var $menu = $("[eve-manager-menu='users'] li:last");
        var $li = $("<li></li>");
        var $a = $("<a href='#'></a>");
        var $name = $("<span>&nbsp;</span>");

        $li.attr("eve-manager-menu","user");
        $li.attr("id",oUser.id);
        $li.data("eveManagerLinkedNode",oNode);
        $name.text(oUser.name);

        $a.append("<span class='ui-icon ui-icon-circle-check'>&nbsp;</span>");
        $a.append($name);
        $li.append($a);
        $menu.after($li);

        //Hide icon
        $a.find(".ui-icon").css("visibility","hidden");

        $("[eve-manager-mainmenu='projectmenu']").menu("refresh");
    },
    removeUser: function(oNode){
        var oUser = oNode.getData();
        var $menu = $("[eve-manager-menu='users'] .users-list");
        //var $user = $menu.find();

        $("#"+ oUser.id).remove();

        $("[eve-manager-mainmenu='projectmenu']").menu("refresh");
    },
    selectUser: function(oNode){
        if(oNode == undefined){
            return;
        }
        var oUser = oNode.getData();

        $("[eve-manager-menu='user']").find(".ui-icon").css("visibility","hidden");
        $("#"+ oUser.id).find(".ui-icon").css("visibility","visible");

        $("[eve-manager-mainmenu='projectmenu']").menu("refresh");
    },

    //Gateways
    addGateway: function(oNode){
        var oGateway = oNode.getData(); 
        var strId = oGateway.id;
        var $element = getGatewayHtmlElement();
        var $gatewayContainer = $("#mygateways .gateways");
        
        $element.attr("id",strId);                                                      //Set html element Id
        $element.data("eveManagerNode",oNode);                                          //Append node to element
        $gatewayContainer.append($element);                                             //Append element
        
        //Set special class to element (eg. ghost class)
        if(oGateway.getParamVal("special") != undefined){
            $element.addClass("special").addClass(oGateway.getParamVal("special"));
        }
        
        //$("#gateway-properties .container").empty();                                    //Empty gateway properties
        return $element;
    },
    removeGateway: function(oNode){
        if(oNode == undefined){
            return;
        }
        
        $("#"+ oNode.getData().id).remove();                                            //Remove html element
    },
    selectGateway: function(oNode){
        var $gatewayContainer = $("#mygateways .gateways");
        var oGateway = oNode.getData();
        
        if($gatewayContainer.length < 1){
            console.warn("User interface are not ready yet");
            return;
        }
        
        var $element = $("#"+ oGateway.id);
        var $oldElement = $gatewayContainer.find(".select");
        
        $oldElement.removeClass("select");                                                          //Remove "select" class
        $oldElement.find(".name").removeClass("ui-state-active");                                   //Remove jquery ui "active" class
        $("#gateway-properties .container").empty();                                                //Empty gateway properties
        
        //Check html element existance
        if($element.length < 1){
            console.error("Any html element has "+ oGateway.id +" id");
            return undefined;
        }
        
        //Check html element existance
        if($element.length > 1){
            console.warn("Wrong id assignment, more than one component has same id: "+ oGateway.id);
        }
        
        $element.addClass("select");                                                                    //Add "select" class to html elements
        $element.find(".name").addClass("ui-state-active");                                             //Add "ui-state-active" class
        $("#gateway-properties .container").loadSync("./components/gateway/gateway.html");              //Fill gateway component
        
        //Jump for special gateways
        if(oGateway.getParamVal("special") != undefined){
            $("#gateway-properties .container").empty();                                                //Empty gateway properties
            return;
        }
        /*if(oGateway.getProtocol().toLowerCase() == "ghost"){
            $("#gateway-properties .container").empty();                                                //Empty gateway properties
            return;
        }*/
        $.publish("/view/gateways/refresh/properties",oGateway);
    },

    //Components
    /** Add component
     *
     * @param oNode
     * @param oParent
     * @returns {jQuery object}
     */
    addComponent: function(oNode,oParent,$parent){

        var oComponent = oNode.getData();


        //var oData = oComponent.getData();
        var strId = oComponent.id;
        var $element;
        if ($parent==undefined)
            $parent = $("#"+ oParent.getData().id);                                                     //Get html parent to append new element
        var type = oComponent.getType();




        if(type == "group") {
            $element = getGroupHtmlElement();                                                           //Get group html element
            $element.accordion(groupAccordion);

            $element.accordion({
                beforeActivate: function (event, ui) {
                   // console.log("before activate");
                    var oNode = ui.newHeader.parent().data("eveManagerNode");
                    // var oNode = $this.data("eveManagerNode");									//Get linked
                    if (ui.newHeader.parent()[0] != undefined) {
                        // evento apertura gruppo
                        application.deleteComponentsInAccordion(oNode, ui.oldPanel.parent());
                        application.selectComponent(oNode,$element);
                        application.fillComponentsInAccordion(oNode, ui.newHeader.parent());

                    }
                    else {
                        // evento chiusura gruppo
                        application.deleteComponentsInAccordion(oNode, ui.oldPanel.parent());
                    }

                },


            });
        }



    /*        $element.accordion({ activate: function(event, ui) {
                console.log("after activate");
                var oNode=ui.newHeader.parent().data("eveManagerNode");
                // var oNode = $this.data("eveManagerNode");									//Get linked node
                application.deleteComponentsInAccordion(oNode,ui.newHeader.parent());
            }
            });
      */      //Create jQuery UI accordion
        else
        {
            $element = getComponentHtmlElement();
            if (oComponent.getParamVal("EVEUIStyle")=="onlyMap")
            {
                $element.addClass("eve-managerui-onlyMap");
            }

            //Get component html element
            $element.find(".components-text").text(type);                                               //Set type as default name
            //$element.find(".icon-set-state_1-64x64").addClass("icon-set-"+ type);                     //Set icon class
            $element.find(".eve-manager-icon").addClass("eve-manager-icon-"+ type);                     //Set icon class
            $element.click(function()
            {
                // evento selezione componente

                var oNode=$element.data("eveManagerNode");
                //console.log("Componente selezionato");
                //console.log(oNode);

                application.selectComponent(oNode,$element);

            });
        }
        
        $element.attr("eve-manager-component",type);                                                    //Set type attribute
        $element.attr("name",oComponent.getParamVal("name"));                             //Set name attribute

        if(!oParent.isRoot()){
            //Check parent type
            if($parent.attr("eve-manager-component") == "group"){
                $parent = $parent.find("> ul");                                                         //Select list as group parent
                $parent.css("padding","5px");
            }
            else{
                //console.error("New component can be added only to groups or root list");                //Error, parent isn't a group or root
                return undefined;
            }
        }
        
        //Append element
        $element.attr("id",strId);                                                                      //Set html element Id
        $element.data("eveManagerNode",oNode);                                                          //Attach node to element

        //TODO In test phase
        //$parent.prepend($element);                                                                      //Append new element
        //$parent.append($element);                                                                       //Append new element
        var frag = document.createDocumentFragment();
        frag.appendChild( $element[0] );
        //$parent[0].appendChild($element[0]);                                                              //Append new element
        $parent.append(frag);     //Append new element

        //$parent[0].appendChild(frag);                                                                       //Append new element

        //Set special class to element (eg. ghost class)
        if(oComponent.getParamVal("special") != undefined){
            $element.addClass(oComponent.getParamVal("special"));
            $element.addClass("special");
        }
        
        //$("#componentproperties .container").empty();                                                 //Empty gateway properties
        //alert($element.data("eveManagerNode"));

        //Check User vew mode
        $element.prop("uservisibility",oComponent.getParamVal("uservisibility"));
        //if(oComponent.getParamVal("uservisibility")){
        $element.attr("em-user-visibility",oComponent.getParamVal("uservisibility"));
        //}


        return $element;
    },
    addComponentInMapList: function(oNode,oParent,$parent){

        var oComponent = oNode.getData();
        //var oData = oComponent.getData();
        var strId = oComponent.id;
        var $element;
        if ($parent==undefined)
            $parent = $("#"+ oParent.getData().id);                                                     //Get html parent to append new element
        var type = oComponent.getType();

        if(type == "group"){
            $element = getGroupHtmlElement();                                                           //Get group html element
            $element.accordion(groupAccordion);                                                         //Create jQuery UI accordion
        }
        else
        {
            $element = getComponentHtmlElement();                                                       //Get component html element
            $element.find(".components-text").text(type);                                               //Set type as default name
            //$element.find(".icon-set-state_1-64x64").addClass("icon-set-"+ type);                     //Set icon class
            $element.find(".eve-manager-icon").addClass("eve-manager-icon-"+ type);                     //Set icon class
        }

        $element.attr("eve-manager-component",type);                                                    //Set type attribute
        $element.attr("name",oComponent.getParamVal("name").toLowerCase());                             //Set name attribute

        if(!oParent.isRoot()){
            //Check parent type
            if($parent.attr("eve-manager-component") == "group"){
                $parent = $parent.find("> ul");                                                         //Select list as group parent
                $parent.css("padding","5px");
            }
            else
            {
                //console.error("New component can be added only to groups or root list");                //Error, parent isn't a group or root
                return undefined;
            }
        }

        //Append element
        $element.attr("id",strId);                                                                      //Set html element Id
        $element.data("eveManagerNode",oNode);                                                          //Attach node to element

        //TODO In test phase
        //$parent.prepend($element);                                                                      //Append new element
        //$parent.append($element);                                                                       //Append new element
        var frag = document.createDocumentFragment();
        frag.appendChild( $element[0] );
        //$parent[0].appendChild($element[0]);                                                              //Append new element
        $parent.append(frag);                                                                       //Append new element
        //$parent[0].appendChild(frag);                                                                       //Append new element

        //Set special class to element (eg. ghost class)
        if(oComponent.getParamVal("special") != undefined){
            $element.addClass(oComponent.getParamVal("special"));
            $element.addClass("special");
        }

        //$("#componentproperties .container").empty();                                                 //Empty gateway properties
        //alert($element.data("eveManagerNode"));

        //Check User vew mode
        $element.prop("uservisibility",oComponent.getParamVal("uservisibility"));
        //if(oComponent.getParamVal("uservisibility")){
        $element.attr("em-user-visibility",oComponent.getParamVal("uservisibility"));
        //}


        return $element;
    },
    /** Remove component
     *
     * @param oNode
     */
    removeComponent: function(oNode){
        if(oNode == undefined){
            return;
        }
        
        $("#"+ oNode.getData().id).remove();                                                            //Remove html element
        $("#componentproperties .container").empty();
    },
    /** Select component
     *
     * @param oNode
     */
    selectComponent: function(oNode){
      //  blr.profiler.begin();
        if (oNode==undefined) return;
        var oComponent = oNode.getData();
        if (oComponent.id=="componentscontainer") return;
        var $element = $("#"+ oComponent.id);

        var strType = $element.attr("eve-manager-component");                           //Get component type
        var $componentcontainer = $("#componentproperties .container");
      //  console.log(oNode);
        if(strType == undefined){
            console.error("String type is undefined");
        }
        
        //Check html element existance
        if($element.length > 1){
            console.warn("Wrong id assignment, more than one component has same id: "+ oComponent.id);
        }

        {
/*
            if(!blr.componentProperties.has(strType)){
                blr.componentProperties.add($componentcontainer,strType,QT.readFile("./components/"+ strType +"/"+ strType +".html"));
            }
            blr.componentProperties.set(strType);
*/

            $componentcontainer.html(QT.readFile("./components/"+ strType +"/"+ strType +".html"));
            //$componentcontainer.html(QT.readFile("./script/buttare_"+ strType +".html"));
            //Hide for special component
            if(oComponent.getParamVal("special") == undefined){
                //$componentcontainer.show();
            }
            else{
                //$componentcontainer.hide();
            }

            //Select element
            $element.addClass("select");
            if($element.attr("eve-manager-component") == "group"){
                $element.find("> h3").addClass("ui-state-active");
            }
            else{
                $element.addClass("ui-state-active");
            }

            //Refresh component's properties area
            $.publish("/view/components/refresh/properties",[oComponent]);
        }

    //    console.log("Select " + blr.profiler.end() + " ms DOM objects " + $('*').length);

        //--console.log("View - Select component ");
        /*
        setTimeout(function(){
            //Load component file
            $componentcontainer.loadSync("./components/"+ strType +"/"+ strType +".html");

            //Hide for special component
            if(oComponent.getParamVal("special") == undefined){
                //$componentcontainer.show();
            }
            else{
                //$componentcontainer.hide();
            }

            //Select element
            $element.addClass("select");
            if($element.attr("eve-manager-component") == "group"){
                $element.find("> h3").addClass("ui-state-active");
            }
            else{
                $element.addClass("select");
                $element.addClass("ui-state-active");
            }

            //Refresh component's properties area
            $.publish("/view/components/refresh/properties",[oComponent]);
        },1);
        */

    },
    deselectComponent: function(){
        //var $element = $("#"+ oNode.getData().id);
        var $element = $("#componentscontainer").find(".select");
        
        if($element.attr("eve-manager-component") == "group"){
            $("#componentscontainer [eve-manager-component] > h3").removeClass("ui-state-active");
        }
        else{
            $("#componentscontainer [eve-manager-component]").removeClass("ui-state-active");
        }
        
        $("#componentscontainer [eve-manager-component]").removeClass("select");                        //Remove "select" class to html elements


        $("#componentproperties .container").children().off();
        $("#componentproperties .container").empty();                                                   //Empty component properties area


        /*$("#componentscontainer [eve-manager-component]").removeClass("select");                      //Remove "select" class to html elements
        $("#componentscontainer [eve-manager-component]").removeClass("ui-state-active");
        $("#componentproperties .container").empty();                                                   //Empty component properties area*/
    },
    clearComponent: function(oNode){
        $("#componentscontainer [eve-manager-component]:not([id])").remove();
    },
    
    //User Interface Components
    addUIComponent: function(oNode,oParent){
        var oUIComponent = oNode.getData();
        var strId = oUIComponent.id;
        var $UILevel=oUIComponent.UILevel;
        //--console.log("UILEvel ->"+$UILevel);
        var $element = getInterfaceHtmlElement();
        var $parent = $("#"+ oParent.getData().id);  
        
                                                           //Get html parent to append new element

        var $parentLevel = $parent.parents(".container").next().find(".level");
        //var type = oUIComponent.getType();
        
        //Append element
        $element.attr("id",strId);                                                                      //Set html element Id
        //$element.attr("eve-manager-interface-type",type);                                               //Set type attribute
        $element.attr("eve-manager-interface-type","icon");                                               //Set type attribute
        $element.data("eveManagerNode",oNode);                                                          //Attach node to element
        $element.find(".name").text(oUIComponent.getParamVal("name"));
        
        var oIcon = oUIComponent.getParamVal("icon");
        
        //var img = new Image();

        var img;
        if ( document.getElementById(oIcon.srcId)==null)
        {
        	oIcon.srcId="app/resource/0";
        }
        img = document.getElementById(oIcon.srcId);

        //Default image loading
        var imgHeight = parseInt(oIcon.setSize.h,10);                        // Image height
        var imgWidth = parseInt(oIcon.setSize.w,10);                         // Image width
        var stretchScale = 1;

        if(oIcon.size.w > 64){
            stretchScale = 0.5;
        }

        var drawHeight = imgHeight * stretchScale;
        var drawWidth = imgWidth * stretchScale;

        var $canvas = $element.find(".icon canvas");
        var canvas = $canvas[0];
        var cont = canvas.getContext("2d");

        $canvas.attr("height",oIcon.size.h*stretchScale);
        $canvas.attr("width",oIcon.size.w*stretchScale);

        //Height - Width streched

        cont.drawImage(img,(oIcon.position.x*stretchScale),(oIcon.position.y*stretchScale),drawWidth,drawHeight);

        var imageData = cont.getImageData(0,0,drawWidth,drawHeight);

        //Color image
        if(oIcon.enableColors){
            //Color image
            for(var j=0; j<imageData.data.length; j+=4){
                imageData.data[j+0] = oIcon.color.R;
                imageData.data[j+1] = oIcon.color.G;
                imageData.data[j+2] = oIcon.color.B;
                imageData.data[j+3] = imageData.data[j+3];
            }
        }

        cont.putImageData(imageData,0,0);


        $element.find(".icon span").remove();
        
        //Check parent and append element
        if(oParent.isRoot()){
            var $container = $parent.find("#level-1");
            $container.find(".drop-target").before($element);

            //$container = $parent.find("#level-11");
            //$container.find(".drop-target").before($element);
        }
        else{
            //$parentLevel.css("border","solid 1px red");

            //Check level
            var deep = oNode.getDeep();

            //var $parentElement = $("#"+ oParent.getData().id);
            //var $container = $parentElement.parents(".container");

            if(deep == 2 && oUIComponent.referenceUId != undefined){
              //  console.log(oUIComponent.referenceUId);
                if (application.getComponentByUid(oUIComponent.referenceUId)!=undefined)
                {
                    var oComponent = application.getComponentByUid(oUIComponent.referenceUId).getData();
                    var type = oComponent.getType();

                    if(type != "scenario"){
                        $parentLevel = $parentLevel.parents(".container").next().find(".level");
                    }

                }
            }

            //$container.find(".drop-target").before($element);
            $parentLevel.find(".drop-target").before($element);
        }
        
        //Set special class to element (eg. ghost class)
        if(oUIComponent.getParamVal("special") != undefined){
            $element.addClass(oUIComponent.getParamVal("special"));
        }


        //Append to level interface
        $parent = $("#mylists").find("[list-id='"+ oParent.getData().id +"']");                                                     //Get html parent to append new element
        $parentLevel = $parent.parents(".container").next().find(".level");
        var $cloned = $element.clone(true,true);

        $cloned.attr("list-id",$cloned.attr("id"));
        $cloned.removeAttr("id");

        //console.error($cloned[0]);
        //console.error("[list-id='"+ oParent.getData().id +"']");
        //console.error($parent.length);

        //Check parent and append element
        if(oParent.isRoot()){
            if ($UILevel==undefined)
            {
                var $container = $("#mylists").find("#level-11");
                oUIComponent.UILevel="level-11";
                $container.find(".drop-target").before($cloned);
            }
            else
            {
                var $container = $("#mylists").find("#level-11");
                $container.find(".drop-target").before($cloned);

            }

        }
        else{

            //$parentLevel.css("border","solid 1px red");

            //Check level
            var deep = oNode.getDeep();


            if(deep == 2 && oUIComponent.referenceUId != undefined){
              //  console.log(oUIComponent.referenceUId);
                if (application.getComponentByUid(oUIComponent.referenceUId)!=undefined)
                {
                    var oComponent = application.getComponentByUid(oUIComponent.referenceUId).getData();
                    var type = oComponent.getType();

                    if(type != "scenario"){
                        $parentLevel = $parentLevel.parents(".container").next().find(".level");
                    }

                }
             }
            if ($UILevel!=undefined)
			    $parentLevel = $("#mylists").find("#"+$UILevel);
            else
                oUIComponent.UILevel="level-1"+deep;
            //$container.find(".drop-target").before($element);
            $parentLevel.find(".drop-target").before($cloned);
        }

        //bigh.cloneCanvas($element.find("canvas")[0]);
        var $oldCanvas = $element.find("canvas")[0];

        $cloned.find(".icon").empty();
        $cloned.find(".icon").append(bigh.cloneCanvas($oldCanvas));
        //$oldCanvas.remove();

        return $element;
    },
    removeUIComponent: function(oNode){
        if(oNode == undefined){
            return;
        }
        
        $("#"+ oNode.getData().id).remove();                                                            //Remove html element
        $("#mylists").find("[list-id='"+ oNode.getData().id +"']").remove();
    },
    selectUIComponent: function(oNode){
        //--console.log("selectUIComponent");
        var oViewer = this;
        var oUIComponent = oNode.getData();
        var oParentNode = oNode.getParent();
        var strId = oUIComponent.id;
        var $element = $("#"+ strId);
        var $element2 = $("#mylists").find("[list-id='"+ strId +"']");
        //var $level = $element.parent();
        var $level = $element2.parent();
        var $nextLevels = $level.parent().nextAll("div").find(".level");
        var $nextLevel = $level.parent().next("div").find(".level");

        //Hide UI Components
        $nextLevels.find("li:not(.drop-target)").hide();

        //Remove all data
        for(var i=0; i < $nextLevels.length; ++i){
            $( $nextLevels[i] ).removeData("eveManagerLinkedNode");
            //$( $nextLevels[i] ).css("border","solid 0px");
        }

        //Set parent node data to current level
        //TODO In test phase
        //$level.data("eveManagerLinkedNode",oParentNode);

        //Show each UI Parents's  childrens
        for(var i=0; i < oParentNode.children.length; ++i){
            var oUIChild = oParentNode.children[i].getData();

            $("#"+ oUIChild.id).show();
        }

        //$level.css("border","solid 1px red");
        //$nextLevel.css("border","solid 1px yellow");

        //Show each UI Component's  childrens
        for(var i=0; i < oNode.children.length; ++i){
            var oUIChild = oNode.children[i].getData();
            $("#"+ oUIChild.id).show();
            $("#mylists").find("[list-id='"+ oUIChild.id +"']").show();
        }

        //Add select class
        $element.addClass("select");
        $element2.addClass("select");

        //Set current node data to next level
        $nextLevel.data("eveManagerLinkedNode",oNode);
        //$nextLevel.css("border","solid 1px yellow");

        //TODO fare i relativi check sui figli dei componenti
        //Check levels
        //if($level.attr("id") == "level-1"){
        if($level.attr("id") == "level-11"){

            //Add parent also to third level
            var $nextNextLevel = $nextLevel.parent().next("div").find(".level");

            $nextNextLevel.data("eveManagerLinkedNode",oNode);
            //$nextNextLevel.css("border","solid 1px red");

            //Check level 2
            if($nextLevel.find("[eve-manager-interface-type]:visible").length < 1 && $nextNextLevel.find("[eve-manager-interface-type]:visible").length > 0){
                $nextLevel.removeData("eveManagerLinkedNode");
                //$nextLevel.css("border","solid 0px");
            }

            //Check level 3 //TODO in test
            if($nextNextLevel.find("[eve-manager-interface-type]:visible").length < 1 && $nextLevel.find("[eve-manager-interface-type]:visible").length > 0){
                $nextNextLevel.removeData("eveManagerLinkedNode");
                //$nextNextLevel.css("border","solid 0px");
            }
        }
        //else if($level.attr("id") == "level-3"){
        else if($level.attr("id") == "level-13"){

            //Add parent also to third level
            var $previousLevel = $level.parent().prev("div").find(".level");

            //Check
            if($previousLevel.find("[eve-manager-interface-type]:visible").length < 1){
                $previousLevel.removeData("eveManagerLinkedNode");
                //$previousLevel.css("border","solid 0px");
            }
        }

        //If component is a scenario remove all data from levels
        //TODO Controllare scenario e se è scenario rimuovere tutti i nodi dai livelli

        //Maps

        //Refresh maps
        var oMapNode = oUIComponent.getMaps();
        var $mapTools = $("#interface-maps .map-tools");
        var $mapTab = $("#interface-maps .maps-container");
        var $mapTabList = $("#interface-maps .maps-container > ul");

        //Html elements
        $mapTools.find(".title").text(oUIComponent.getParamVal("name"));            //Name

        //Empty old content
        $mapTabList.find("li:not(.add-tab)").remove();
        $mapTab.find(".ui-tabs-panel").remove();

        //Generate tabs
        for(var i=0; i<oMapNode.children.length; ++i){
            var oMap = oMapNode.children[i].getData();

            oViewer.addMap(oMapNode.children[i]);
        }

        $mapTab.tabs("refresh");

        //Set viewer as droppable
        $mapTab.find(".viewer").droppable({
            accept: "#interface-elements *",
            activeClass:"active"
        });

        //Set elements as draggable
        var aGrid = (presenter.preferences.getInterfacePreference("gridEnabled"))?presenter.preferences.getInterfacePreference("grid"):false;

        /*console.error(aGrid);
        console.error(mapElementsDraggable);
        console.error($.extend(false, mapElementsDraggable, { grid: aGrid } ));*/

        $mapTab.find(".viewer > *").draggable($.extend(true, mapElementsDraggable, { grid:aGrid } ));
        $mapTab.find(".viewer > *").draggable($.extend(true, mapElementsDraggable, { grid:aGrid } ));
        //$mapTab.find(".viewer > *").draggable($.extend(false, mapElementsDraggable, { grid:aGrid } ));

        //$("#interface-maps .maps-container").tabs("option","active",0); //Select always first tab

        if(oMapNode.children.length > 0){
            $("#interface-maps").show();                                    //Show Map Editor
        }
        else{
            $("#interface-maps").hide();                                    //Hide Map Editor
        }

        $.publish("/view/interface/maps/resize");                           //Resize also maps

    },
    deselectUIComponent: function(oNode){
        if(oNode == undefined){
            return;
        }
        
        var oUIComponent = oNode.getData();
        var $element = $("#"+ oUIComponent.id);
        var $containers = $element.parents(".container").nextAll();
        var $levels = $(".level:not(#level-1)");

        /*for(var i=0; i<$containers.length; ++i){
            $( $containers[i]).find(".level").removeData("eveManagerLinkedNode");
        }*/

        /*console.error($container.find(".level").length);
        console.log($container.next().find(".level").length);
        console.log($container.next().next().find(".level").length);
        console.log($container.next().next().next().find(".level").length);*/

        /*for(var i=0; i<$levels.length; ++i){
            $( $levels[i]).removeData("eveManagerLinkedNode");
        }*/
        
        //Remove from html element "select" class
        $element.removeClass("select");

        $("#mylists").find("[list-id='"+ oUIComponent.id +"']").removeClass("select");

        
        //HIQOO Map Editor
        $("#interface-maps").hide();                                    //Hide Map Editor
    },
    
    //Maps
    addMap: function(oNode){
        //Check if is a Map
        if(! (oNode.getData() instanceof Map) ){
            console.error("Node data is not instance of Map");
            return;
        }
        
        var oMap = oNode.getData();
        var oTemplate = getMapHtmlElements();
        var $tabHeader = oTemplate.header;
        var $tabContents = oTemplate.contents;
        var $mapTab = $("#interface-maps .maps-container");
        var $mapTabList = $mapTab.find("> ul");
        
        //Set html building parameters
        $tabHeader.attr("title",oMap.name);
        $tabHeader.find("a").attr("href","#"+ oMap.id).text(oMap.name);
        $tabHeader.data("MapNodeData",oNode);
        $tabHeader.data("eveManagerNode",oNode);
        
        $tabContents.attr("id",oMap.id);
        $tabContents.data("MapNodeData",oNode);
        $mapTabList.find(".add-tab").before($tabHeader);
        
        //Append tab
        $mapTab.append($tabContents);
        
        //Refresh tabs
        $mapTab.tabs("refresh");
        
        //Set viewer as droppable
        var oUser = application.getSelectUser().getData();
        var $viewer = $mapTab.find(".viewer");

        $viewer.height(oUser.canvas.h);
        $viewer.width(oUser.canvas.w);
        $viewer.droppable({
            accept: "#interface-elements *",
            activeClass:"active"
        });
        
        //Set elements as draggable
        $mapTab.find(".viewer > *").draggable($.extend(false,mapElementsDraggable));
        //console.error([ $viewer.offset().top, $viewer.offset().left, $viewer.width(), $viewer.height() ]);
        
        $.publish("/view/interface/maps/resize");                       //Resize also maps
    },
    removeMap: function(oNode){
        //Check oNode
        if(oNode == undefined){
            console.error("Node is undefined");
            return;
        }
        
        //Check if is a Map
        if(! (oNode.getData() instanceof Map) ){
            console.error("Node data is not instance of Map");
            return;
        }
        
        var oMap = oNode.getData();
        var $map = $("#interface-maps .maps-container");
        var $mapTabList = $map.find("> ul");
        var $mapTabAnchor = $mapTabList.find("[href='#"+ oMap.id +"']").parent();
        var $tabContents = $("#"+ oMap.id);                                             //Get map linked tab panel
        
        $mapTabAnchor.remove();
        $tabContents.remove();
        
        //Refresh tabs if has all tabs
        if($mapTabList.find("> li:not(.add-tab)").length){
            $map.tabs("refresh");
        }
    },
    selectMap: function(oNode){
        var oView = this;
        var oMap = oNode.getData();
        var $mapTools = $("#interface-maps .map-tools");                                //Get map toolbar
        var $tabContents = $("#"+ oMap.id);                                             //Get map linked tab panel
        var $viewer = $tabContents.find(".viewer");                                     //Get viewer
        var $mapsContainer = $("#interface-maps .maps-container");
        var $tabAnchors = $mapsContainer.find("> ul li:not(.add-tab)");
        var $selectTabAnchor = $mapsContainer.find("> ul [href='#"+ oMap.id +"']").parent();
        
        $mapTools.find("#imageVisible").prop("checked",oMap.imageVisible);              //Refresh background image visibility
        $mapTools.find("#mapTransparent").prop("checked",oMap.trasparent);              //Refresh background color
        
        //Refresh color
        $mapTools.find("#mapChangeColor").css("background-color",bigh.rgb2hex(oMap.color.R,oMap.color.G,oMap.color.B));

        $mapTools.find("#mapEnableGrid").prop("checked",presenter.preferences.getInterfacePreference("gridEnabled"));              //Refresh snap enable
        $mapTools.find("#mapGrid").val(presenter.preferences.getInterfacePreference("grid")[0]);                          //Refresh snap enable
        $("#mapGrid").prop("disabled",!presenter.preferences.getInterfacePreference("gridEnabled"));

        $mapTools.find("#mapEnableSnap").prop("checked",presenter.preferences.getInterfacePreference("snapEnabled"));
        
        //Select a Tab if it isn't already selected
        
        var nNewActive = $tabAnchors.index($selectTabAnchor);
        
        $mapsContainer.tabs("option","active",nNewActive);                              //Select tab
        
        //console.error("nNewActive: "+ nNewActive);
        
        //Map parameters for view
        $.publish("/view/uicomponent/map/"+ oMap.id +"/refresh/parameter",[oMap,"trasparent"]);
        $.publish("/view/uicomponent/map/"+ oMap.id +"/refresh/parameter",[oMap,"imageVisible"]);
        $.publish("/view/uicomponent/map/"+ oMap.id +"/refresh/parameter",[oMap,"color"]);
        $.publish("/view/uicomponent/map/"+ oMap.id +"/refresh/parameter",[oMap,"background"]);

        //Draw components
        for(var i=0; i<oMap.components.length; ++i){
        //      var oMapComponent = oMap.components[i].getData();
        //      console.log(oMap.components[i]);
        //      console.log(oNode);
            oView.addMapComponent(oMap.components[i],oNode);
        }
    },
    deselectMap: function(oNode){
        //Check node
        if(oNode == undefined){
            return;
        }
        
        //Check map
        if(! (oNode.getData() instanceof Map) ){
            console.error("Node data is not instance of Map");
            return;
        }
        
        var oMap = oNode.getData();
        var $mapTools = $("#interface-maps .map-tools");                                //Get map toolbar
        var $tabContents = $("#"+ oMap.id);                                             //Get map linked tab panel
        var $viewer = $tabContents.find(".viewer");                                     //Get viewer
        var $mapsContainer = $("#interface-maps .maps-container");
        var $tabAnchors = $mapsContainer.find("> ul li:not(.add-tab)");
        var $selectTabAnchor = $mapsContainer.find("> ul [href='#"+ oMap.id +"']").parent();
        
        //Deselect Map
        $mapsContainer.tabs("option","active",false);
        
        //Empty viewer
        $viewer.empty();
    },
    
    //Maps Components
    addMapComponent: function(oNode,oMapNode){
        //Check
        if(oNode == undefined){
            console.error("Map component's node is undefined");
            return;
        }
        if(oMapNode == undefined){
            console.error("Map's node is undefined");
            return;
        }
        
        var oMapComponent = oNode.getData();
        var oComponent = oMapComponent.getReference().getData();


        var oMap = oMapNode.getData();

        var aIcons = oComponent.getParamVal("icons"); // prendo le icone dal componente originale
        if (oMapComponent.iconChanged) // se è stato cambiato prendo quelle cambiate
            aIcons = oMapComponent["icons"];
        var $tabContents = $("#"+ oMap.id);
        var $viewer = $tabContents.find(".viewer");
        var $mapComponent = getMapElementHtml();

        
        //Check if is visible
        if(!$tabContents.is(":visible")){
            return;
        }
        
        //Append component
        $viewer.append($mapComponent);
        
        //Set data
        $mapComponent.data("MapComponentRef",oMapComponent);
        $mapComponent.data("eveManagerNode",oNode);
        
        $mapComponent.draggable($.extend(false,mapElementsDraggable,{ refreshPositions: true }) );
        
        $mapComponent.attr("id",oMapComponent.id);
        $mapComponent.css("top", oMapComponent.position.y);
        $mapComponent.css("left", oMapComponent.position.x);
        $mapComponent.attr("title",oMapComponent.name);
       // $mapComponent.find(".name").text(oMapComponent.name);
        $mapComponent.find(".label").text((oComponent.getParamVal("gui/label")!=undefined)?oComponent.getParamVal("gui/label"):oMapComponent.name);

        //Draw icon
        var oIcon;
        var $canvas = $mapComponent.find(".icon");
        var $label = $mapComponent.find(".label");

        //Get first icon
        for(var k in aIcons){
            oIcon = aIcons[k];
            break;
        }
        
        //Set icon dimension
        //$mapComponent.css("height", oIcon.size.h);
        //$mapComponent.css("width", oIcon.size.w);
        if (oIcon==undefined) return undefined;
        oIcon.srcId="app/resource/0"; // fixme: // rimuovere dopo problema icone
        var img = document.getElementById(oIcon.srcId);
        //var img = document.getElementById("./css/icons-set/images/icons-state_1-128x128.png");
        
        //Default image loading
     //   console.log(oIcon.srcId);

        var imgHeight = parseInt(img.height,10);                        // Image height
        var imgWidth = parseInt(img.width,10);                          // Image width
        var stretchScale = oIcon.scaleFactor;                           //Get icon scale factor
        var stretchScaleWidth = oMapComponent.scaleFactorWidth;         //Get icon scale factor Width
        var stretchScaleHeight = oMapComponent.scaleFactorHeight;       //Get icon scale factor Height



        $mapComponent.resizable({
            containment: ".viewer",
            minWidth: 30,
            minHeight: 30,

            handles: "n, e, s, w",
            resize: function (event, ui)
            {
                var oRef=ui.element.data().MapComponentRef;
                var w=ui.size.width;
                var h=ui.size.height;

                oRef.scaleFactorWidth=w/oIcon.size.w;
                oRef.scaleFactorHeight=h/oIcon.size.h;

                /*
                 //Scale factor Width
                 $.publish("/model/uicomponent/map/mapcomponent/"+ oRef.id +"/update/parameter",[oRef,"scaleFactorWidth",w/oIcon.size.w]);
                 $.publish("/view/uicomponent/map/mapcomponent/"+ oRef.id +"/refresh/parameter",[oRef,"scaleFactorWidth"]);

                 //Scale factor Height
                 $.publish("/model/uicomponent/map/mapcomponent/"+ oRef.id +"/update/parameter",[oRef,"scaleFactorHeight",h/oIcon.size.h]);
                 $.publish("/view/uicomponent/map/mapcomponent/"+ oRef.id +"/refresh/parameter",[oRef,"scaleFactorHeight"]);
                 */

            },
            stop: function (event, ui)
            {
                var oRef=ui.element.data().MapComponentRef;
                var w=ui.size.width;
                var h=ui.size.height;

                oRef.scaleFactorWidth=w/oIcon.size.w;
                oRef.scaleFactorHeight=h/oIcon.size.h;


                 //Scale factor Width
                 $.publish("/model/uicomponent/map/mapcomponent/"+ oRef.id +"/update/parameter",[oRef,"scaleFactorWidth",w/oIcon.size.w/oMapComponent.scaleFactor]);
                // $.publish("/view/uicomponent/map/mapcomponent/"+ oRef.id +"/refresh/parameter",[oRef,"scaleFactorWidth"]);

                 //Scale factor Height
                 $.publish("/model/uicomponent/map/mapcomponent/"+ oRef.id +"/update/parameter",[oRef,"scaleFactorHeight",h/oIcon.size.h/oMapComponent.scaleFactor]);
                 $.publish("/view/uicomponent/map/mapcomponent/"+ oRef.id +"/refresh/parameter",[oRef,"scaleFactorHeight"]);


            }

        });

        //Check existance
        if(oMapComponent.scaleFactorWidth == undefined){
            stretchScaleWidth=1;
        }
        if(oMapComponent.scaleFactorHeight == undefined){
            stretchScaleHeight=1;
        }

        /*
            if(oMapComponent.scaleFactor == undefined){
                var preferences = application.getConfigParam("preferences");
                oMapComponent.scaleFactor = preferences.interface.iconsScaleFactor;
                //oMapComponent.scaleFactor = 1;
        }*/
        if (stretchScale==undefined)
            stretchScale=1;
        stretchScale *= oMapComponent.scaleFactor;                      //Scale icon for map

        var drawHeight = imgHeight * stretchScale;
        var drawWidth = imgWidth * stretchScale;
        var canvas = $canvas[0];
        var cont = canvas.getContext("2d");

        //console.log(oIcon);
        $canvas.attr("height",oIcon.size.h * stretchScaleHeight * stretchScale);
        $canvas.attr("width",oIcon.size.w * stretchScaleWidth * stretchScale);

        $label.css("zoom",oMapComponent.scaleFactor);
        
        //Height - Width streched
        //alert(drawWidth + " x " + drawHeight);
        //cont.drawImage(img,(oIcon.position.x * stretchScale),(oIcon.position.y * stretchScale),drawWidth,drawHeight);

        // cerchio di background delle icone nelle mappe

        if (!oMapComponent.uiDisableCircle)
        {
            cont.beginPath();
            var color="";
            if (oIcon.enableColors)
                color= "rgba("+oIcon.color.R+", "+oIcon.color.G+", "+oIcon.color.B+", 0.8)";
            else
                color = "rgba(0,0,0, 0.8)";

            var xCentre=canvas.width/2;
            var yCentre=canvas.height/2;
            var rX=canvas.width*1.3;
            var rY=canvas.height;

            drawEllipse(cont,xCentre,yCentre,rX,rY,color);
   /*         cont.arc(oIcon.size.h * stretchScaleHeight * stretchScale/2,oIcon.size.w * stretchScaleWidth * stretchScale/2,oIcon.size.w * stretchScaleWidth * stretchScale/2-3, 0, 2 * Math.PI, false);


            cont.fill();
            cont.lineWidth = 1;
            cont.strokeStyle = '#444444';
            cont.stroke();

            var canvas = document.getElementById('myCanvas');
            var cont = canvas.getContext('2d');
          */

        }
        cont.drawImage(
            img,
            (oIcon.position.x * stretchScaleWidth * stretchScale),
            (oIcon.position.y * stretchScaleHeight * stretchScale),
            drawWidth*stretchScaleWidth,
            drawHeight*stretchScaleHeight
        );




        //var imageData = cont.getImageData(0,0,drawWidth,drawHeight);
        //var imageData = cont.getImageData(0,0,128,128);
        //var imageData = cont.getImageData(0,0,oIcon.size.w,oIcon.size.h);
        var imageData;
        try {
            imageData = cont.getImageData(0, 0, oIcon.size.w * stretchScaleWidth * stretchScale, oIcon.size.h * stretchScaleHeight * stretchScale);
        }
        catch (ex)
        {
            console.log(ex);
        }
        //Color image
        if(oIcon.enableColors && oMapComponent.uiDisableCircle){
            //console.error(oIcon.color);
            //Color image
            for(var j=0; j<imageData.data.length; j+=4){
                imageData.data[j+0] = oIcon.color.R;
                imageData.data[j+1] = oIcon.color.G;
                imageData.data[j+2] = oIcon.color.B;
                imageData.data[j+3] = imageData.data[j+3];
            }
        }
        
        cont.putImageData(imageData,0,0);

        //Set icon dimension
        //$mapComponent.height($canvas.attr("height"));
        //$mapComponent.css("height","auto");
        $mapComponent.width($canvas.attr("width"));

        switch(oMapComponent["viewmode"]){
            case 2:
                $mapComponent.find(".icon").hide();
                $mapComponent.find(".name").hide();
                $mapComponent.find(".label").show();
                //$mapComponent.css("height","auto");
                //alert($mapComponent.find(".label").attr("height"));
                $mapComponent.height($mapComponent.find(".label").height());
                //$mapComponent.width("auto");
                //$mapComponent.width($mapComponent.find(".label").attr("width"));
                $mapComponent.css("width","auto");
                if(oIcon.enableColors)
                {
                    $mapComponent.find(".label").css('color','rgb('+oIcon.color.R+','+oIcon.color.G+','+oIcon.color.B+')');
                }

                break;
            case 10:
                $mapComponent.find(".icon").remove();
                $mapComponent.find(".label").remove();

                var $img = $("<img />");

                // only first time calculate the size based on the image size
                if (oMapComponent.firstTimeCreation)
                {
                    console.log("first");
                    oMapComponent.firstTimeCreation=false;
                     var imageLoadedWidth=oMapComponent.getReference().getData().getParamVal("gui/width");
                     var imageLoadedHeight=oMapComponent.getReference().getData().getParamVal("gui/height");
                     oMapComponent.scaleFactorWidth=imageLoadedWidth/imageLoadedHeight;

                     var stretchScaleWidth = oMapComponent.scaleFactorWidth;         //Get icon scale factor Width
                     var stretchScaleHeight = oMapComponent.scaleFactorHeight;       //Get icon scale factor Height

                }
                  $img.attr("height","100%");//oIcon.size.h * stretchScaleHeight * stretchScale);
                $img.attr("width","100%");//oIcon.size.w * stretchScaleWidth * stretchScale);

                $img.attr("src",oComponent.getParamVal("gui/src"));



                //$mapComponent.css("height","auto");
                //alert($mapComponent.find(".label").attr("height"));
                //$mapComponent.height($mapComponent.find(".label").height());
                //$mapComponent.width("auto");
                //$mapComponent.width($mapComponent.find(".label").attr("width"));
                //$mapComponent.css("width","auto");
                $mapComponent.width(oIcon.size.w * stretchScaleWidth * stretchScale);
                $mapComponent.height(oIcon.size.h * stretchScaleHeight * stretchScale);
                $mapComponent.append($img);
                break;
            default:
                $mapComponent.find(".icon").show();
                $mapComponent.find(".name").show();
                $mapComponent.find(".label").hide();
                $mapComponent.height($canvas.attr("height"));
                //$mapComponent.width($canvas.attr("width"));
                //$mapComponent.width($canvas.attr("width"));
        }


        return $mapComponent;
    },
    removeMapComponent: function(oNode,oMapNode){
        var oMapComponent = oNode.getData();
        var oMap = oMapNode.getData();
        var $map = $("#"+ oMap.id);
        
        //Check visibility
        if(!$map.is(":visible") || $map.length < 1){
            return;
        }
        
        var $element = $("#"+ oMapComponent.id);
        $element.remove();
    },
    selectMapComponent: function(oNode){
        var oMapComponent = oNode.getData();

        var $element = $("#"+ oMapComponent.id+":visible");
        $element.addClass("selected").focus();
    },
    deselectMapComponent: function(oNode,oMapNode){
        var $elements = $("#interface-maps .viewer:visible .map-element");
        $elements.removeClass("selected");
    },
    addRelation: function(oNode){
        var oRelation = oNode.getData();
        var $element = getRelationElementHtml();

        //$element.removeClass("incomplete");
        //$element.find(".source").removeClass("ui-widget-content").addClass("complete").text(sSrc);
        //$element.find(".destination").removeClass("ui-widget-content").removeClass("ui-state-disabled").addClass("complete").text(sDst);

        $element.data("eveManagerNode",oNode);
        $element.attr("id",oRelation.id);


        $("#myrelations > .relations-container").append($element);

        $element.find(".destination, .source").droppable(relationDroppable);

        //Check relation params
        if(oRelation.getParamVal("checkTime") != undefined){
            $.publish("/view/relations/"+ oRelation.id +"/refresh/parameter",[oRelation,"checkTime"]);

        }
        if(oRelation.getParamVal("source") != undefined){
            $.publish("/view/relations/"+ oRelation.id +"/refresh/parameter",[oRelation,"source"]);
        }
        if(oRelation.getParamVal("destination") != undefined){
            $.publish("/view/relations/"+ oRelation.id +"/refresh/parameter",[oRelation,"destination"]);
        }
    },
    removeRelation: function(oNode){
        $("#"+ oNode.getData().id).remove();
    },

    //Resources
    addIconsPan: function(oPan){
        var iconSet = new Image();
        iconSet.src = oPan["params"].src;
        iconSet.id = oPan["id"];
        
        $(iconSet).hide();
        $("body").append(iconSet);
        
        //Create
        if(!oPan["params"]["visible"]){
            return;
        }
        
        var $iconsSet = $("#interface-icons");
        //var $iconsSet = $("#interface-icons, #lists-icons");
        var $list = $("<div class='icons'><ul></ul></div>");
        
        $iconsSet.append("<h3><span>"+ oPan["params"]["label"] +"</span></h3>");
        $iconsSet.append($list);
        
        var $list = $list.find("ul");
        var scale = 1;
        
        if(oPan["params"]["iconsWidth"] > 64){
            scale = 0.5;
        }
        
        $list.data("iconsPan",oPan);
        $list.attr("application-pan-id",oPan["id"]);
        $list.attr("application-pan-name",oPan["params"]["label"]);
        $list.attr("application-pan-scale",scale);
        
        for(var i=0; i<oPan["params"]["_positions"].length; ++i){
            var $icon = $("<span>&nbsp;</span>");
            var $li = $("<li></li>");
            
            $icon.addClass("icon");
            
            $icon.css("display","inline-block");
            $icon.css("width",(oPan["params"]["iconsWidth"] * scale) +"px");
            $icon.css("height",(oPan["params"]["iconsHeight"] * scale) +"px");
            $icon.css("background-repeat","no-repeat");
            $icon.css("background-size",(oPan["params"]["imgWidth"] * scale) +"px "+ (oPan["params"]["imgHeight"] * scale) +"px");
            $icon.css("background-image","url('"+ oPan["params"]["src"] +"')");
            $icon.css("background-position",(oPan["params"]["_positions"][i]["x"] * scale) +"px "+ (oPan["params"]["_positions"][i]["y"] * scale) +"px");
            
            $li.attr("application-icon-index");
            $li.attr("application-pan-id",oPan["id"]);
            $li.attr("application-pan-scale",scale);
            
            $li.append($icon);
            $list.append($li);
        }

        //Add icons to second interface tab

        var $iconsSet = $("#lists-icons");
        //var $iconsSet = $("#interface-icons, #lists-icons");
        var $list = $("<div class='icons'><ul></ul></div>");

        $iconsSet.append("<h3><span>"+ oPan["params"]["label"] +"</span></h3>");
        $iconsSet.append($list);

        var $list = $list.find("ul");
        
        var scale = 1;

        if(oPan["params"]["iconsWidth"] > 64){
            scale = 0.5;
        }

        $list.data("iconsPan",oPan);
        $list.attr("application-pan-id",oPan["id"]);
        $list.attr("application-pan-name",oPan["params"]["name"]);
        $list.attr("application-pan-scale",scale);

        for(var i=0; i<oPan["params"]["_positions"].length; ++i){
            var $icon = $("<span>&nbsp;</span>");
            var $li = $("<li></li>");

            $icon.addClass("icon");

            $icon.css("display","inline-block");
            $icon.css("width",(oPan["params"]["iconsWidth"] * scale) +"px");
            $icon.css("height",(oPan["params"]["iconsHeight"] * scale) +"px");
            $icon.css("background-repeat","no-repeat");
            $icon.css("background-size",(oPan["params"]["imgWidth"] * scale) +"px "+ (oPan["params"]["imgHeight"] * scale) +"px");
            $icon.css("background-image","url('"+ oPan["params"]["src"] +"')");
            $icon.css("background-position",(oPan["params"]["_positions"][i]["x"] * scale) +"px "+ (oPan["params"]["_positions"][i]["y"] * scale) +"px");

            $li.attr("application-icon-index");
            $li.attr("application-pan-id",oPan["id"]);
            $li.attr("application-pan-scale",scale);

            $li.append($icon);
            $list.append($li);
        }
        
        //Publish icons update
        $.publish("/view/application/refresh/icons");
    },
    
    //View utilities
    resetView: function(){
        //Empty Users
        $("[eve-manager-menu='users'] [eve-manager-menu='user']").remove();


        //Empty interfce levels
        $("#myinterface .level > *:not(.drop-target)").remove();
        $("#mylists .level > *:not(.drop-target)").remove();

        //Empty relations
        $("#myrelations .relations-container").empty();

        //Empty maps viewer
        $("#interface-maps").hide();
        
        $("#componentscontainer, #componentproperties .container, #mygateways > .gateways, #gateway-properties .container").empty();
    }
};